/*
 * Copyright (C) 2001-2003 Stephane Magnenat & Julien Pilet.  
 * For any question or comment contact us at nct@ysagoon.com
 * or leto@calodox.org
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*! \file
 * GDB remote protocol for jelie
 * \author Julien Pilet
 * \date november 2002
 */

#ifndef _GDBREMOTE_H_
#define _GDBREMOTE_H_

#include <assert.h>
#include "tcp.h"
#include <map>

#define GDB_REMOTE_DEFAULT_PORT 3141
#define GDB_REMOTE_BUFFER_SIZE (32 * 1024)

//! Assembled value for the instruction: bkpt #0
#define BKPT_0_INSTR 0xe1200070 

class JTAGpxa250;

/*! \brief Glue between the GDB remote protocol and jelie.
 *
 * This class interprets the gdb protocol and translates the command it
 * receives into JTAG order for the target processor.
 */
class GdbRemote { 
public:

	GdbRemote(JTAGpxa250 *pxa);
	~GdbRemote();

	//! open a port for accepting new TCP connections
	//! \param port TCP port to listen to
	//! \return true on sucess, false on error
	bool openPort(int port = GDB_REMOTE_DEFAULT_PORT);

	//! accept new incoming connection or listen to active connection
	//! \return the largest file descriptor added, to use with select.
	int prepareFD_SET(fd_set *set);

	//! process new connections or messages sent
	void processFD_SET(fd_set *set);

	//! prints a string describing the last error on stderr
	void printError();

private:
	JTAGpxa250 *pxa;
	int clientSocket;	
	bool running;
	unsigned int hardBreak[2];

	//! contains data sent by GDB
	unsigned char buffer[GDB_REMOTE_BUFFER_SIZE];	

	//! contains the message to send
	char message[GDB_REMOTE_BUFFER_SIZE];
	int bufferLength;
	int ptr;
	void reset() { ptr = 0; }
	unsigned char current() { return buffer[ptr]; }
	bool next() { ++ptr; assert(ptr<GDB_REMOTE_BUFFER_SIZE); return ptr < bufferLength; }

	void sendPacket();
	void parse();
	void parsePacket();
	void parseValidPacket();
	static unsigned char toAsciiHex(unsigned int v);
	static unsigned char asciiToHex(unsigned char c);
	static void hexPrintWord(unsigned char *str, unsigned int val);
	unsigned int parseHexWord();
	unsigned int parseHexByte();

	static void callback(void *p);

	void resumeExec();
	void query();
	void lastSignal();
	void setThread();
	void getRegisters();
	void setRegisters();
	void readMem();
	void writeMem();
	void cont();
	void insertBreakWatch();
	void removeBreakWatch();
	void step();
};

#endif // _GDB_REMOTE_H_

